@@ -1,6 +1,6 @@
 (***********************************************************************)
 (*                                                                     *)
-(*                           Objective Caml                            *)
+(*                                OCaml                                *)
 (*                                                                     *)
 (*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         *)
 (*                                                                     *)
@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: arch.ml 7784 2007-01-01 13:07:35Z xleroy $ *)
+(* $Id: arch.ml 12187 2012-02-24 10:13:02Z xleroy $ *)
 
 (* Machine-specific command-line options *)
 
@@ -51,6 +51,10 @@ let size_addr = 8
 let size_int = 8
 let size_float = 8
 
+(* Behavior of division *)
+
+let division_crashes_on_overflow = true
+
 (* Operations on addressing modes *)
 
 let identity_addressing = Iindexed 0
