@@ -1,9 +1,9 @@
 (***********************************************************************)
 (*                                                                     *)
-(*                           Objective Caml                            *)
+(*                                OCaml                                *)
 (*                                                                     *)
 (*          Jerome Vouillon, projet Cristal, INRIA Rocquencourt        *)
-(*          Objective Caml port by John Malecki and Xavier Leroy       *)
+(*          OCaml port by John Malecki and Xavier Leroy                *)
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
@@ -11,10 +11,8 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: main.ml,v 1.21 2008/07/29 08:31:41 xleroy Exp $ *)
+(* $Id: main.ml 12511 2012-05-30 13:29:48Z lefessan $ *)
 
-open Primitives
-open Misc
 open Input_handling
 open Question
 open Command_line
@@ -26,6 +24,7 @@ open Program_management
 open Frames
 open Show_information
 open Format
+open Primitives
 
 let line_buffer = Lexing.from_function read_user_input
 
@@ -47,12 +46,12 @@ let rec protect ppf restart loop =
           !current_checkpoint.c_pid;
         pp_print_flush ppf ();
         stop_user_input ();
-        loop ppf)
+        restart ppf)
   | Toplevel ->
       protect ppf restart (function ppf ->
         pp_print_flush ppf ();
         stop_user_input ();
-        loop ppf)
+        restart ppf)
   | Sys.Break ->
       protect ppf restart (function ppf ->
         fprintf ppf "Interrupted.@.";
@@ -62,7 +61,7 @@ let rec protect ppf restart loop =
             try_select_frame 0;
             show_current_event ppf;
           end);
-        loop ppf)
+        restart ppf)
   | Current_checkpoint_lost ->
       protect ppf restart (function ppf ->
         fprintf ppf "Trying to recover...@.";
@@ -70,7 +69,7 @@ let rec protect ppf restart loop =
         recover ();
         try_select_frame 0;
         show_current_event ppf;
-        loop ppf)
+        restart ppf)
   | Current_checkpoint_lost_start_at (time, init_duration) ->
       protect ppf restart (function ppf ->
         let b =
@@ -109,7 +108,39 @@ let rec protect ppf restart loop =
       kill_program ();
       raise x
 
-let toplevel_loop () = protect Format.std_formatter loop loop
+let execute_file_if_any () =
+  let buffer = Buffer.create 128 in
+  begin
+    try
+      let base = ".ocamldebug" in
+      let file =
+        if Sys.file_exists base then
+          base
+        else
+          Filename.concat (Sys.getenv "HOME") base in
+      let ch = open_in file in
+      fprintf Format.std_formatter "Executing file %s@." file;
+      while true do
+        let line = string_trim (input_line ch) in
+        if line <> ""  && line.[0] <> '#' then begin
+          Buffer.add_string buffer line;
+          Buffer.add_char buffer '\n'
+        end
+      done;
+    with _ -> ()
+  end;
+  let len = Buffer.length buffer in
+  if len > 0 then
+    let commands = Buffer.sub buffer 0 (pred len) in
+    line_loop Format.std_formatter (Lexing.from_string commands)
+
+let toplevel_loop () =
+  interactif := false;
+  current_prompt := "";
+  execute_file_if_any ();
+  interactif := true;
+  current_prompt := debugger_prompt;
+  protect Format.std_formatter loop loop
 
 (* Parsing of command-line arguments *)
 
@@ -127,7 +158,11 @@ let set_checkpoints n =
 let set_directory dir =
   Sys.chdir dir
 let print_version () =
-  printf "The Objective Caml debugger, version %s@." Sys.ocaml_version;
+  printf "The OCaml debugger, version %s@." Sys.ocaml_version;
+  exit 0;
+;;
+let print_version_num () =
+  printf "%s@." Sys.ocaml_version;
   exit 0;
 ;;
 
@@ -144,13 +179,19 @@ let speclist = [
       "<filename>  Set the name of the communication socket";
    "-version", Arg.Unit print_version,
       " Print version and exit";
+   "-vnum", Arg.Unit print_version_num,
+      " Print version number and exit";
    ]
 
+let function_placeholder () =
+  raise Not_found
+
 let main () =
+  Callback.register "Debugger.function_placeholder" function_placeholder;
   try
-    socket_name := 
+    socket_name :=
       (match Sys.os_type with
-        "Win32" -> 
+        "Win32" ->
           (Unix.string_of_inet_addr Unix.inet_addr_loopback)^
           ":"^
           (string_of_int (10000 + ((Unix.getpid ()) mod 10000)))
@@ -169,8 +210,7 @@ let main () =
         arguments := !arguments ^ " " ^ (Filename.quote Sys.argv.(j))
       done
     end;
-    current_prompt := debugger_prompt;
-    printf "\tObjective Caml Debugger version %s@.@." Config.version;
+    printf "\tOCaml Debugger version %s@.@." Config.version;
     Config.load_path := !default_load_path;
     Clflags.recursive_types := true;    (* Allow recursive types. *)
     toplevel_loop ();                   (* Toplevel. *)
@@ -184,6 +224,11 @@ let main () =
       Env.report_error err_formatter e;
       eprintf "@]@.";
       exit 2
+  | Cmi_format.Error e ->
+      eprintf "Debugger [version %s] environment error:@ @[@;" Config.version;
+      Cmi_format.report_error err_formatter e;
+      eprintf "@]@.";
+      exit 2
 
 let _ =
   Printexc.catch (Unix.handle_unix_error main) ()
