@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: odoc_analyse.ml 10355 2010-05-03 15:06:17Z guesdon $ *)
+(* $Id: odoc_analyse.ml 12862 2012-08-16 09:44:48Z guesdon $ *)
 
 (** Analysis of source files. This module is strongly inspired from
     driver/main.ml :-) *)
@@ -46,7 +46,7 @@ let preprocess sourcefile =
   match !Clflags.preprocessor with
     None -> sourcefile
   | Some pp ->
-      let tmpfile = Filename.temp_file "camlpp" "" in
+      let tmpfile = Filename.temp_file "ocamldocpp" "" in
       let comm = Printf.sprintf "%s %s > %s" pp sourcefile tmpfile in
       if Ccomp.command comm <> 0 then begin
         remove_file tmpfile;
@@ -73,15 +73,14 @@ let parse_file inputfile parse_fun ast_magic =
   let ic = open_in_bin inputfile in
   let is_ast_file =
     try
-      let buffer = String.create (String.length ast_magic) in
-      really_input ic buffer 0 (String.length ast_magic);
+      let buffer = Misc.input_bytes ic (String.length ast_magic) in
       if buffer = ast_magic then true
       else if String.sub buffer 0 9 = String.sub ast_magic 0 9 then
         raise Outdated_version
       else false
     with
       Outdated_version ->
-        fatal_error "Ocaml and preprocessor have incompatible versions"
+        fatal_error "OCaml and preprocessor have incompatible versions"
     | _ -> false
   in
   let ast =
@@ -114,7 +113,10 @@ let process_implementation_file ppf sourcefile =
   let env = initial_env () in
   try
     let parsetree = parse_file inputfile Parse.implementation ast_impl_magic_number in
-    let typedtree = Typemod.type_implementation sourcefile prefixname modulename env parsetree in
+    let typedtree =
+      Typemod.type_implementation
+        sourcefile prefixname modulename env parsetree
+    in
     (Some (parsetree, typedtree), inputfile)
   with
     e ->
@@ -165,6 +167,9 @@ let process_error exn =
   | Env.Error err ->
       Location.print_error_cur_file ppf;
       Env.report_error ppf err
+  | Cmi_format.Error err ->
+      Location.print_error_cur_file ppf;
+      Cmi_format.report_error ppf err
   | Ctype.Tags(l, l') ->
       Location.print_error_cur_file ppf;
       fprintf ppf
@@ -203,18 +208,18 @@ let process_error exn =
 
 (** Process the given file, according to its extension. Return the Module.t created, if any.*)
 let process_file ppf sourcefile =
-  if !Odoc_args.verbose then
+  if !Odoc_global.verbose then
     (
      let f = match sourcefile with
-       Odoc_args.Impl_file f
-     | Odoc_args.Intf_file f -> f
-     | Odoc_args.Text_file f -> f
+       Odoc_global.Impl_file f
+     | Odoc_global.Intf_file f -> f
+     | Odoc_global.Text_file f -> f
      in
      print_string (Odoc_messages.analysing f) ;
      print_newline ();
     );
   match sourcefile with
-    Odoc_args.Impl_file file ->
+    Odoc_global.Impl_file file ->
       (
        Location.input_name := file;
        try
@@ -228,7 +233,7 @@ let process_file ppf sourcefile =
              in
              file_module.Odoc_module.m_top_deps <- Odoc_dep.impl_dependencies parsetree ;
 
-             if !Odoc_args.verbose then
+             if !Odoc_global.verbose then
                (
                 print_string Odoc_messages.ok;
                 print_newline ()
@@ -246,18 +251,18 @@ let process_file ppf sourcefile =
            incr Odoc_global.errors ;
            None
       )
-  | Odoc_args.Intf_file file ->
+  | Odoc_global.Intf_file file ->
       (
        Location.input_name := file;
        try
          let (ast, signat, input_file) = process_interface_file ppf file in
          let file_module = Sig_analyser.analyse_signature file
-             !Location.input_name ast signat
+             !Location.input_name ast signat.sig_type
          in
 
          file_module.Odoc_module.m_top_deps <- Odoc_dep.intf_dependencies ast ;
 
-         if !Odoc_args.verbose then
+         if !Odoc_global.verbose then
            (
             print_string Odoc_messages.ok;
             print_newline ()
@@ -275,11 +280,15 @@ let process_file ppf sourcefile =
            incr Odoc_global.errors ;
            None
       )
-  | Odoc_args.Text_file file ->
+  | Odoc_global.Text_file file ->
       Location.input_name := file;
       try
         let mod_name =
-          String.capitalize (Filename.basename (Filename.chop_extension file))
+          let s =
+            try Filename.chop_extension file
+            with _ -> file
+          in
+          String.capitalize (Filename.basename s)
         in
         let txt =
           try Odoc_text.Texter.text_of_string (Odoc_misc.input_file_as_string file)
@@ -289,7 +298,7 @@ let process_file ppf sourcefile =
         let m =
           {
             Odoc_module.m_name = mod_name ;
-            Odoc_module.m_type = Types.Tmty_signature [] ;
+            Odoc_module.m_type = Types.Mty_signature [] ;
             Odoc_module.m_info = None ;
             Odoc_module.m_is_interface = true ;
             Odoc_module.m_file = file ;
@@ -297,7 +306,7 @@ let process_file ppf sourcefile =
               [Odoc_module.Element_module_comment txt] ;
             Odoc_module.m_loc =
               { Odoc_types.loc_impl = None ;
-                Odoc_types.loc_inter = Some (file, 0) } ;
+                Odoc_types.loc_inter = Some (Location.in_file file) } ;
             Odoc_module.m_top_deps = [] ;
             Odoc_module.m_code = None ;
             Odoc_module.m_code_intf = None ;
@@ -474,20 +483,20 @@ let analyse_files ?(init=[]) files =
   in
   (* Remove elements between the stop special comments, if needed. *)
   let modules =
-    if !Odoc_args.no_stop then
+    if !Odoc_global.no_stop then
       modules_pre
     else
       remove_elements_between_stop modules_pre
   in
 
 
-  if !Odoc_args.verbose then
+  if !Odoc_global.verbose then
     (
      print_string Odoc_messages.merging;
      print_newline ()
     );
-  let merged_modules = Odoc_merge.merge !Odoc_args.merge_options modules in
-  if !Odoc_args.verbose then
+  let merged_modules = Odoc_merge.merge !Odoc_global.merge_options modules in
+  if !Odoc_global.verbose then
     (
      print_string Odoc_messages.ok;
      print_newline ();
@@ -499,20 +508,20 @@ let analyse_files ?(init=[]) files =
        merged_modules
     )
   in
-  if !Odoc_args.verbose then
+  if !Odoc_global.verbose then
     (
      print_string Odoc_messages.cross_referencing;
      print_newline ()
     );
   let _ = Odoc_cross.associate modules_list in
 
-  if !Odoc_args.verbose then
+  if !Odoc_global.verbose then
     (
      print_string Odoc_messages.ok;
      print_newline ();
     );
 
-  if !Odoc_args.sort_modules then
+  if !Odoc_global.sort_modules then
     Sort.list (fun m1 -> fun m2 -> m1.Odoc_module.m_name < m2.Odoc_module.m_name) merged_modules
   else
     merged_modules
