@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: odoc_merge.ml,v 1.10.12.2 2006/08/10 12:34:02 guesdon Exp $ *)
+(* $Id: odoc_merge.ml 11123 2011-07-20 09:17:07Z doligez $ *)
 
 (** Merge of information from [.ml] and [.mli] for a module.*)
 
@@ -23,6 +23,28 @@ open Odoc_exception
 open Odoc_class
 open Odoc_module
 
+let merge_before_tags l =
+  let rec iter acc = function
+    [] -> List.rev acc
+  | (v, text) :: q ->
+      let (l1, l2) = List.partition
+        (fun (v2,_) -> v = v2) q
+      in
+      let acc =
+        let text =
+          List.fold_left
+            (fun acc t -> acc @ [Raw " "] @ t)
+            text (List.map snd l1)
+        in
+        (v, text) :: acc
+      in
+      iter acc l2
+  in
+  iter [] l
+;;
+
+let version_separators = Str.regexp "[\\.\\+]";;
+
 (** Merge two Odoctypes.info struture, completing the information of
    the first one with the information in the second one.
    The merge treatment depends on a given merge_option list.
@@ -83,6 +105,19 @@ let merge_info merge_options (m1 : info) (m2 : info) =
         else
           Some v1
   in
+  let new_before =
+    match m1.i_before, m2.i_before with
+      [], [] -> []
+    | l, []
+    | [], l -> l
+    | l1, l2 ->
+        if List.mem Merge_before merge_options then
+          merge_before_tags (m1.i_before @ m2.i_before)
+        else
+          l1 in
+  let new_before = List.map (fun (v, t) -> (Str.split version_separators v, v, t)) new_before in
+  let new_before = List.sort Pervasives.compare new_before in
+  let new_before = List.map (fun (_, v, t) -> (v, t)) new_before in
   let new_dep =
     match m1.i_deprecated, m2.i_deprecated with
       None, None -> None
@@ -170,6 +205,7 @@ let merge_info merge_options (m1 : info) (m2 : info) =
     Odoc_types.i_version = new_version ;
     Odoc_types.i_sees = new_sees ;
     Odoc_types.i_since = new_since ;
+    Odoc_types.i_before = new_before ;
     Odoc_types.i_deprecated = new_dep ;
     Odoc_types.i_params = new_params ;
     Odoc_types.i_raised_exceptions = new_raised_exceptions ;
@@ -196,7 +232,7 @@ let merge_types merge_options mli ml =
     Type_abstract, _ ->
       ()
 
-  | Type_variant (l1, _), Type_variant (l2, _) ->
+  | Type_variant l1, Type_variant l2 ->
       let f cons =
         try
           let cons2 = List.find
@@ -217,14 +253,14 @@ let merge_types merge_options mli ml =
           cons.vc_text <- new_desc
         with
           Not_found ->
-            if !Odoc_args.inverse_merge_ml_mli then
+            if !Odoc_global.inverse_merge_ml_mli then
               ()
             else
               raise (Failure (Odoc_messages.different_types mli.ty_name))
       in
       List.iter f l1
 
-  | Type_record (l1, _), Type_record (l2, _) ->
+  | Type_record l1, Type_record l2 ->
       let f record =
         try
           let record2= List.find
@@ -245,7 +281,7 @@ let merge_types merge_options mli ml =
           record.rf_text <- new_desc
         with
           Not_found ->
-            if !Odoc_args.inverse_merge_ml_mli then
+            if !Odoc_global.inverse_merge_ml_mli then
               ()
             else
               raise (Failure (Odoc_messages.different_types mli.ty_name))
@@ -253,7 +289,7 @@ let merge_types merge_options mli ml =
       List.iter f l1
 
   | _ ->
-      if !Odoc_args.inverse_merge_ml_mli then
+      if !Odoc_global.inverse_merge_ml_mli then
         ()
       else
         raise (Failure (Odoc_messages.different_types mli.ty_name))
@@ -321,7 +357,7 @@ let merge_classes merge_options mli ml =
                      a.att_value.val_info <- merge_info_opt merge_options
                          a.att_value.val_info a2.att_value.val_info;
                      a.att_value.val_loc <- { a.att_value.val_loc with loc_impl = a2.att_value.val_loc.loc_impl } ;
-                     if !Odoc_args.keep_code then
+                     if !Odoc_global.keep_code then
                        a.att_value.val_code <- a2.att_value.val_code;
                      true
                     )
@@ -360,7 +396,7 @@ let merge_classes merge_options mli ml =
                         parameters because the associated comment of a parameter may have been changed by the merge.*)
                      Odoc_value.update_value_parameters_text m.met_value;
 
-                     if !Odoc_args.keep_code then
+                     if !Odoc_global.keep_code then
                        m.met_value.val_code <- m2.met_value.val_code;
 
                      true
@@ -398,7 +434,7 @@ let merge_class_types merge_options mli ml =
                      a.att_value.val_info <- merge_info_opt merge_options
                          a.att_value.val_info a2.att_value.val_info;
                      a.att_value.val_loc <- { a.att_value.val_loc with loc_impl = a2.att_value.val_loc.loc_impl } ;
-                     if !Odoc_args.keep_code then
+                     if !Odoc_global.keep_code then
                        a.att_value.val_code <- a2.att_value.val_code;
 
                      true
@@ -437,7 +473,7 @@ let merge_class_types merge_options mli ml =
                         parameters because the associated comment of a parameter may have been changed y the merge.*)
                      Odoc_value.update_value_parameters_text m.met_value;
 
-                     if !Odoc_args.keep_code then
+                     if !Odoc_global.keep_code then
                        m.met_value.val_code <- m2.met_value.val_code;
 
                      true
@@ -529,7 +565,7 @@ let rec merge_module_types merge_options mli ml =
                 Element_module m2 ->
                   if m2.m_name = m.m_name then
                     (
-                     ignore(merge_modules merge_options m m2);
+                     ignore (merge_modules merge_options m m2);
 (*
                      m.m_info <- merge_info_opt merge_options m.m_info m2.m_info;
                      m.m_loc <- { m.m_loc with loc_impl = m2.m_loc.loc_impl } ;
@@ -601,7 +637,7 @@ let rec merge_module_types merge_options mli ml =
                         parameters because the associated comment of a parameter may have been changed y the merge.*)
                      Odoc_value.update_value_parameters_text v;
 
-                     if !Odoc_args.keep_code then
+                     if !Odoc_global.keep_code then
                        v.val_code <- v2.val_code;
 
                      true
@@ -691,20 +727,20 @@ and merge_modules merge_options mli ml =
   mli.m_top_deps <- remove_doubles mli.m_top_deps ml.m_top_deps ;
 
   let code =
-    if !Odoc_args.keep_code then
+    if !Odoc_global.keep_code then
       match mli.m_code, ml.m_code with
-	Some s, _ -> Some s
-      |	_, Some s -> Some s
-      |	_ -> None
+        Some s, _ -> Some s
+      | _, Some s -> Some s
+      | _ -> None
     else
       None
   in
   let code_intf =
-    if !Odoc_args.keep_code then
+    if !Odoc_global.keep_code then
       match mli.m_code_intf, ml.m_code_intf with
-	Some s, _ -> Some s
-      |	_, Some s -> Some s
-      |	_ -> None
+        Some s, _ -> Some s
+      | _, Some s -> Some s
+      | _ -> None
     else
       None
   in
@@ -777,7 +813,7 @@ and merge_modules merge_options mli ml =
                 Element_module m2 ->
                   if m2.m_name = m.m_name then
                     (
-                     ignore(merge_modules merge_options m m2);
+                     ignore (merge_modules merge_options m m2);
 (*
                      m.m_info <- merge_info_opt merge_options m.m_info m2.m_info;
                      m.m_loc <- { m.m_loc with loc_impl = m2.m_loc.loc_impl } ;
@@ -847,7 +883,7 @@ and merge_modules merge_options mli ml =
                     parameters because the associated comment of a parameter may have been changed y the merge.*)
                  Odoc_value.update_value_parameters_text v;
 
-                 if !Odoc_args.keep_code then
+                 if !Odoc_global.keep_code then
                    v.val_code <- v2.val_code;
                  true
                 )
@@ -939,19 +975,19 @@ let merge merge_options modules_list =
             (
              (* we can merge m with m2 if there is an implementation
                 and an interface.*)
-             let f b = if !Odoc_args.inverse_merge_ml_mli then not b else b in
+             let f b = if !Odoc_global.inverse_merge_ml_mli then not b else b in
              match f m.m_is_interface, f m2.m_is_interface with
                true, false -> (merge_modules merge_options m m2) :: (iter l_others)
              | false, true -> (merge_modules merge_options m2 m) :: (iter l_others)
              | false, false ->
-                 if !Odoc_args.inverse_merge_ml_mli then
+                 if !Odoc_global.inverse_merge_ml_mli then
                    (* two Module.ts for the .mli ! *)
                    raise (Failure (Odoc_messages.two_interfaces m.m_name))
                  else
                    (* two Module.t for the .ml ! *)
                    raise (Failure (Odoc_messages.two_implementations m.m_name))
              | true, true ->
-                 if !Odoc_args.inverse_merge_ml_mli then
+                 if !Odoc_global.inverse_merge_ml_mli then
                    (* two Module.t for the .ml ! *)
                    raise (Failure (Odoc_messages.two_implementations m.m_name))
                  else
@@ -964,5 +1000,3 @@ let merge merge_options modules_list =
 
   in
   iter modules_list
-
-(* eof $Id: odoc_merge.ml,v 1.10.12.2 2006/08/10 12:34:02 guesdon Exp $ *)
