@@ -1,6 +1,6 @@
 (***********************************************************************)
 (*                                                                     *)
-(*                           Objective Caml                            *)
+(*                                OCaml                                *)
 (*                                                                     *)
 (*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         *)
 (*                                                                     *)
@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: str.ml,v 1.19.10.1 2005/11/07 15:59:04 doligez Exp $ *)
+(* $Id: str.ml 11156 2011-07-27 14:17:02Z doligez $ *)
 
 (** String utilities *)
 
@@ -29,7 +29,7 @@ module Charset =
   struct
     type t = string (* of length 32 *)
 
-    let empty = String.make 32 '\000'
+    (*let empty = String.make 32 '\000'*)
     let full = String.make 32 '\255'
 
     let make_empty () = String.make 32 '\000'
@@ -44,9 +44,9 @@ module Charset =
     let singleton c =
       let s = make_empty () in add s c; s
 
-    let range c1 c2 =
+    (*let range c1 c2 =
       let s = make_empty () in add_range s c1 c2; s
-
+    *)
     let complement s =
       let r = String.create 32 in
       for i = 0 to 31 do
@@ -96,7 +96,7 @@ module Charset =
 type re_syntax =
     Char of char
   | String of string
-  | CharClass of Charset.t
+  | CharClass of Charset.t * bool  (* true = complemented, false = normal *)
   | Seq of re_syntax list
   | Alt of re_syntax * re_syntax
   | Star of re_syntax
@@ -156,7 +156,7 @@ let displ dest from = dest - from - 1
 let rec is_nullable = function
     Char c -> false
   | String s -> s = ""
-  | CharClass cl -> false
+  | CharClass(cl, cmpl) -> false
   | Seq rl -> List.for_all is_nullable rl
   | Alt (r1, r2) -> is_nullable r1 || is_nullable r2
   | Star r -> true
@@ -175,7 +175,7 @@ let rec is_nullable = function
 let rec first = function
     Char c -> Charset.singleton c
   | String s -> if s = "" then Charset.full else Charset.singleton s.[0]
-  | CharClass cl -> cl
+  | CharClass(cl, cmpl) -> if cmpl then Charset.complement cl else cl
   | Seq rl -> first_seq rl
   | Alt (r1, r2) -> Charset.union (first r1) (first r2)
   | Star r -> Charset.full
@@ -197,12 +197,13 @@ and first_seq = function
 (* Transform a Char or CharClass regexp into a character class *)
 
 let charclass_of_regexp fold_case re =
-  let cl =
+  let (cl1, compl) =
     match re with
-      Char c -> Charset.singleton c
-    | CharClass cl -> cl
+    | Char c -> (Charset.singleton c, false)
+    | CharClass(cl, compl) -> (cl, compl)
     | _ -> assert false in
-  if fold_case then Charset.fold_case cl else cl
+  let cl2 = if fold_case then Charset.fold_case cl1 else cl1 in
+  if compl then Charset.complement cl2 else cl2
 
 (* The case fold table: maps characters to their lowercase equivalent *)
 
@@ -289,9 +290,10 @@ let compile fold_case re =
           else
             emit_instr op_STRING (cpool_index s)
       end
-  | CharClass cl ->
-      let cl' = if fold_case then Charset.fold_case cl else cl in
-      emit_instr op_CHARCLASS (cpool_index cl')
+  | CharClass(cl, compl) ->
+      let cl1 = if fold_case then Charset.fold_case cl else cl in
+      let cl2 = if compl then Charset.complement cl1 else cl1 in
+      emit_instr op_CHARCLASS (cpool_index cl2)
   | Seq rl ->
       emit_seq_code rl
   | Alt(r1, r2) ->
@@ -365,14 +367,14 @@ let compile fold_case re =
       emit_code r;
       let lbl = !progpos in
       patch_instr pos_pushback op_PUSHBACK lbl
-  | Group(n, r) -> 
+  | Group(n, r) ->
       if n >= 32 then failwith "too many \\(...\\) groups";
       emit_instr op_BEGGROUP n;
       emit_code r;
       emit_instr op_ENDGROUP n;
       numgroups := max !numgroups (n+1)
   | Refgroup n ->
-      emit_instr op_REFGROUP n      
+      emit_instr op_REFGROUP n
   | Bol ->
       emit_instr op_BOL 0
   | Eol ->
@@ -492,10 +494,11 @@ let parse s =
   and regexp3 i =
     match s.[i] with
       '\\' -> regexpbackslash (i+1)
-    | '['  -> let (c, j) = regexpclass0 (i+1) in (CharClass c, j)
+    | '['  -> let (c, compl, j) = regexpclass0 (i+1) in
+              (CharClass(c, compl), j)
     | '^'  -> (Bol, i+1)
     | '$'  -> (Eol, i+1)
-    | '.'  -> (CharClass dotclass, i+1)
+    | '.'  -> (CharClass(dotclass, false), i+1)
     | c    -> (Char c, i+1)
   and regexpbackslash i =
     if i >= len then (Char '\\', i) else
@@ -520,8 +523,8 @@ let parse s =
           (Char c, i + 1)
   and regexpclass0 i =
     if i < len && s.[i] = '^'
-    then let (c, j) = regexpclass1 (i+1) in (Charset.complement c, j)
-    else regexpclass1 i
+    then let (c, j) = regexpclass1 (i+1) in (c, true, j)
+    else let (c, j) = regexpclass1 i in (c, false, j)
   and regexpclass1 i =
     let c = Charset.make_empty() in
     let j = regexpclass2 c i i in
@@ -562,9 +565,9 @@ let quote s =
   done;
   String.sub buf 0 !pos
 
-let regexp_string s = compile false (String s) 
+let regexp_string s = compile false (String s)
 
-let regexp_string_case_fold s = compile true (String s) 
+let regexp_string_case_fold s = compile true (String s)
 
 (** Matching functions **)
 
@@ -639,86 +642,99 @@ let replace_matched repl matched =
 let substitute_first expr repl_fun text =
   try
     let pos = search_forward expr text 0 in
-    String.concat "" [string_before text pos; 
+    String.concat "" [string_before text pos;
                       repl_fun text;
                       string_after text (match_end())]
   with Not_found ->
     text
 
+let opt_search_forward re s pos =
+  try Some(search_forward re s pos) with Not_found -> None
+
 let global_substitute expr repl_fun text =
-  let rec replace start last_was_empty =
-    try
-      let startpos = if last_was_empty then start + 1 else start in
-      if startpos > String.length text then raise Not_found;
-      let pos = search_forward expr text startpos in
-      let end_pos = match_end() in
-      let repl_text = repl_fun text in
-      String.sub text start (pos-start) ::
-      repl_text ::
-      replace end_pos (end_pos = pos)
-    with Not_found ->
-      [string_after text start] in
-  String.concat "" (replace 0 false)
+  let rec replace accu start last_was_empty =
+    let startpos = if last_was_empty then start + 1 else start in
+    if startpos > String.length text then
+      string_after text start :: accu
+    else
+      match opt_search_forward expr text startpos with
+      | None ->
+          string_after text start :: accu
+      | Some pos ->
+          let end_pos = match_end() in
+          let repl_text = repl_fun text in
+          replace (repl_text :: String.sub text start (pos-start) :: accu)
+                  end_pos (end_pos = pos)
+  in
+    String.concat "" (List.rev (replace [] 0 false))
 
 let global_replace expr repl text =
   global_substitute expr (replace_matched repl) text
 and replace_first expr repl text =
-  substitute_first expr (replace_matched repl) text  
+  substitute_first expr (replace_matched repl) text
 
 (** Splitting *)
 
-let search_forward_progress expr text start =
-  let pos = search_forward expr text start in
-  if match_end() = start && start < String.length text
-  then search_forward expr text (start + 1)
-  else pos
+let opt_search_forward_progress expr text start =
+  match opt_search_forward expr text start with
+  | None -> None
+  | Some pos ->
+      if match_end() > start then
+        Some pos
+      else if start < String.length text then
+        opt_search_forward expr text (start + 1)
+      else None
 
 let bounded_split expr text num =
   let start =
     if string_match expr text 0 then match_end() else 0 in
-  let rec split start n =
-    if start >= String.length text then [] else
-    if n = 1 then [string_after text start] else
-      try
-        let pos = search_forward_progress expr text start in
-        String.sub text start (pos-start) :: split (match_end()) (n-1)
-      with Not_found ->
-        [string_after text start] in
-  split start num
+  let rec split accu start n =
+    if start >= String.length text then accu else
+    if n = 1 then string_after text start :: accu else
+      match opt_search_forward_progress expr text start with
+      | None ->
+          string_after text start :: accu
+      | Some pos ->
+          split (String.sub text start (pos-start) :: accu)
+                (match_end()) (n-1)
+  in
+    List.rev (split [] start num)
 
 let split expr text = bounded_split expr text 0
 
 let bounded_split_delim expr text num =
-  let rec split start n =
-    if start > String.length text then [] else
-    if n = 1 then [string_after text start] else
-      try
-        let pos = search_forward_progress expr text start in
-        String.sub text start (pos-start) :: split (match_end()) (n-1)
-      with Not_found ->
-        [string_after text start] in
-  if text = "" then [] else split 0 num
+  let rec split accu start n =
+    if start > String.length text then accu else
+    if n = 1 then string_after text start :: accu else
+      match opt_search_forward_progress expr text start with
+      | None ->
+          string_after text start :: accu
+      | Some pos ->
+          split (String.sub text start (pos-start) :: accu)
+                (match_end()) (n-1)
+  in
+    if text = "" then [] else List.rev (split [] 0 num)
 
 let split_delim expr text = bounded_split_delim expr text 0
 
 type split_result = Text of string | Delim of string
 
 let bounded_full_split expr text num =
-  let rec split start n =
-    if start >= String.length text then [] else
-    if n = 1 then [Text(string_after text start)] else
-      try
-        let pos = search_forward_progress expr text start in
-        let s = matched_string text in
-        if pos > start then
-          Text(String.sub text start (pos-start)) ::
-          Delim(s) ::
-          split (match_end()) (n-1)
-        else
-          Delim(s) ::
-          split (match_end()) (n-1)
-      with Not_found ->
-        [Text(string_after text start)] in
-  split 0 num
+  let rec split accu start n =
+    if start >= String.length text then accu else
+    if n = 1 then Text(string_after text start) :: accu else
+      match opt_search_forward_progress expr text start with
+      | None ->
+          Text(string_after text start) :: accu
+      | Some pos ->
+          let s = matched_string text in
+          if pos > start then
+            split (Delim(s) :: Text(String.sub text start (pos-start)) :: accu)
+                  (match_end()) (n-1)
+          else
+            split (Delim(s) :: accu)
+                  (match_end()) (n-1)
+  in
+    List.rev (split [] 0 num)
 
 let full_split expr text = bounded_full_split expr text 0
