@@ -1,6 +1,6 @@
 (***********************************************************************)
 (*                                                                     *)
-(*                           Objective Caml                            *)
+(*                                OCaml                                *)
 (*                                                                     *)
 (*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         *)
 (*                                                                     *)
@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: asttypes.mli 10250 2010-04-08 03:58:41Z garrigue $ *)
+(* $Id: asttypes.mli 12511 2012-05-30 13:29:48Z lefessan $ *)
 
 (* Auxiliary a.s.t. types used by parsetree and typedtree. *)
 
@@ -38,3 +38,8 @@ type override_flag = Override | Fresh
 type closed_flag = Closed | Open
 
 type label = string
+
+type 'a loc = 'a Location.loc = {
+  txt : 'a;
+  loc : Location.t;
+}
