@@ -1,6 +1,6 @@
 (***********************************************************************)
 (*                                                                     *)
-(*                           Objective Caml                            *)
+(*                                OCaml                                *)
 (*                                                                     *)
 (*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         *)
 (*                                                                     *)
@@ -10,19 +10,20 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: lexer.mli,v 1.14 2003/11/21 16:01:13 xleroy Exp $ *)
+(* $Id: lexer.mli 12511 2012-05-30 13:29:48Z lefessan $ *)
 
 (* The lexical analyzer *)
 
+val init : unit -> unit
 val token: Lexing.lexbuf -> Parser.token
 val skip_sharp_bang: Lexing.lexbuf -> unit
 
 type error =
   | Illegal_character of char
   | Illegal_escape of string
-  | Unterminated_comment
+  | Unterminated_comment of Location.t
   | Unterminated_string
-  | Unterminated_string_in_comment
+  | Unterminated_string_in_comment of Location.t
   | Keyword_as_label of string
   | Literal_overflow of string
 ;;
@@ -34,3 +35,9 @@ open Format
 val report_error: formatter -> error -> unit
 
 val in_comment : unit -> bool;;
+val in_string : unit -> bool;;
+
+
+val print_warnings : bool ref
+val comments : unit -> (string * Location.t) list
+val token_with_comments : Lexing.lexbuf -> Parser.token
